@@ -10,7 +10,8 @@ Marshal values of types that make up 'Block' elements.
 -}
 module Text.Pandoc.Lua.Marshal.Block
   ( -- * Single Block elements
-    peekBlock
+    typeBlock
+  , peekBlock
   , peekBlockFuzzy
   , pushBlock
     -- * List of Blocks
@@ -28,6 +29,7 @@ module Text.Pandoc.Lua.Marshal.Block
 import Control.Applicative ((<|>), optional)
 import Control.Monad.Catch (throwM)
 import Control.Monad ((<$!>))
+import Data.Aeson (encode)
 import Data.Data (showConstr, toConstr)
 import Data.Maybe (fromMaybe)
 import Data.Proxy (Proxy (Proxy))
@@ -83,6 +85,20 @@ pushBlocks xs = do
       <#> parameter peekFilter "Filter" "lua_filter" "table of filter functions"
       =#> functionResult pushBlocks "Blocks" "modified list"
     rawset (nth 3)
+
+    pushName "__tostring"
+    pushDocumentedFunction $ lambda
+      ### liftPure show
+      <#> parameter peekBlocksFuzzy "Blocks" "self" ""
+      =#> functionResult pushString "string" "native Haskell representation"
+    rawset (nth 3)
+
+    pushName "__tojson"
+    pushDocumentedFunction $ lambda
+      ### liftPure encode
+      <#> parameter peekBlocksFuzzy "Blocks" "self" ""
+      =#> functionResult pushLazyByteString "string" "JSON representation"
+    rawset (nth 3)
   setmetatable (nth 2)
 {-# INLINABLE pushBlocks #-}
 
@@ -120,17 +136,23 @@ typeBlock = deftype "Block"
     ### liftPure show
     <#> udparam typeBlock "self" ""
     =#> functionResult pushString "string" "Haskell representation"
+  , operation (CustomOperation "__tojson") $ lambda
+    ### liftPure encode
+    <#> udparam typeBlock "self" ""
+    =#> functionResult pushLazyByteString "string" "JSON representation"
   ]
   [ possibleProperty "attr" "element attributes"
       (pushAttr, \case
           CodeBlock attr _     -> Actual attr
           Div attr _           -> Actual attr
+          Figure attr _ _      -> Actual attr
           Header _ attr _      -> Actual attr
           Table attr _ _ _ _ _ -> Actual attr
           _                    -> Absent)
       (peekAttr, \case
           CodeBlock _ code     -> Actual . flip CodeBlock code
           Div _ blks           -> Actual . flip Div blks
+          Figure _ capt blks   -> Actual . (\attr -> Figure attr capt blks)
           Header lvl _ blks    -> Actual . (\attr -> Header lvl attr blks)
           Table _ c cs h bs f  -> Actual . (\attr -> Table attr c cs h bs f)
           _                    -> const Absent)
@@ -142,8 +164,12 @@ typeBlock = deftype "Block"
           Table attr c cs h _ f -> Actual . (\bs -> Table attr c cs h bs f)
           _                     -> const Absent)
   , possibleProperty "caption" "element caption"
-      (pushCaption, \case {Table _ capt _ _ _ _ -> Actual capt; _ -> Absent})
+      (pushCaption, \case
+          Figure _ capt _      -> Actual capt
+          Table _ capt _ _ _ _ -> Actual capt
+          _ -> Absent)
       (peekCaptionFuzzy, \case
+          Figure attr _ blks     -> Actual . (\c -> Figure attr c blks)
           Table attr _ cs h bs f -> Actual . (\c -> Table attr c cs h bs f)
           _                      -> const Absent)
   , possibleProperty "colspecs" "column alignments and widths"
@@ -222,9 +248,10 @@ getBlockContent = \case
   Para inlns          -> Actual $ ContentInlines inlns
   Plain inlns         -> Actual $ ContentInlines inlns
   Header _ _ inlns    -> Actual $ ContentInlines inlns
-  -- inline content
+  -- block content
   BlockQuote blks     -> Actual $ ContentBlocks blks
   Div _ blks          -> Actual $ ContentBlocks blks
+  Figure _ _ blks     -> Actual $ ContentBlocks blks
   -- lines content
   LineBlock lns       -> Actual $ ContentLines lns
   -- list items content
@@ -244,6 +271,7 @@ setBlockContent _ = \case
   -- block content
   BlockQuote _     -> Actual . BlockQuote . blockContent
   Div attr _       -> Actual . Div attr . blockContent
+  Figure attr c _  -> Actual . Figure attr c . blockContent
   -- lines content
   LineBlock _      -> Actual . LineBlock . lineContent
   -- list items content
@@ -324,6 +352,16 @@ blockConstructors =
     <#> optAttrParam
     =#> blockResult "Div element"
 
+  , defun "Figure"
+    ### liftPure3 (\content mcapt mattr ->
+                     let attr = fromMaybe nullAttr mattr
+                         capt = fromMaybe (Caption mempty mempty) mcapt
+                     in Figure attr capt content)
+    <#> parameter peekBlocksFuzzy "Blocks" "content" "figure content"
+    <#> opt (parameter peekCaptionFuzzy "Caption" "caption" "figure caption")
+    <#> optAttrParam
+    =#> blockResult "Figure element"
+
   , defun "Header"
     ### liftPure3 (\lvl content mattr ->
                      Header lvl (fromMaybe nullAttr mattr) content)
@@ -341,10 +379,6 @@ blockConstructors =
     <#> parameter (peekList peekInlinesFuzzy) "{Inlines,...}" "content" "lines"
     =#> blockResult "LineBlock element"
 
-  , defun "Null"
-    ### return Null
-    =#> blockResult "Null element"
-
   , defun "OrderedList"
     ### liftPure2 (\items mListAttrib ->
                      let defListAttrib = (1, DefaultStyle, DefaultDelim)
