@@ -1,6 +1,6 @@
 {-# LANGUAGE OverloadedStrings    #-}
 {- |
-Copyright               : © 2021-2022 Albert Krewinkel
+Copyright               : © 2021-2023 Albert Krewinkel
 SPDX-License-Identifier : MIT
 Maintainer              : Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>
 
@@ -8,7 +8,8 @@ Marshaling/unmarshaling functions of 'Pandoc' values.
 -}
 module Text.Pandoc.Lua.Marshal.Pandoc
   ( -- * Pandoc
-    peekPandoc
+    typePandoc
+  , peekPandoc
   , pushPandoc
   , mkPandoc
     -- * Meta
@@ -21,6 +22,7 @@ module Text.Pandoc.Lua.Marshal.Pandoc
 
 import Control.Applicative (optional)
 import Control.Monad ((<$!>))
+import Data.Aeson (encode)
 import Data.Maybe (fromMaybe)
 import HsLua
 import Text.Pandoc.Lua.Marshal.Block (peekBlocksFuzzy, pushBlocks)
@@ -55,6 +57,10 @@ typePandoc = deftype "Pandoc"
     ### liftPure show
     <#> parameter peekPandoc "Pandoc" "doc" ""
     =#> functionResult pushString "string" "native Haskell representation"
+  , operation (CustomOperation "__tojson") $ lambda
+    ### liftPure encode
+    <#> udparam typePandoc "self" ""
+    =#> functionResult pushLazyByteString "string" "JSON representation"
   ]
   [ property "blocks" "list of blocks"
       (pushBlocks, \(Pandoc _ blks) -> blks)
